#version 150

const float outlineThickness = 0.004; // thickness of the outline as ratio of the fov
const float outlineIntensity = 1; // how intense the outline is (0 = invisible, 1 = pure white)
const float onePixelDistance = 62.0; // the outline becomes thinner over distance, at this distance it has the size of a single pixel
const float distanceFalloff = 0.001; // intensity falloff over distance, i dont recommend changing this value unless you are rendering some stuff thats really far away

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D TranslucentSampler;
uniform sampler2D TranslucentDepthSampler;
uniform sampler2D ItemEntitySampler;
uniform sampler2D ItemEntityDepthSampler;
uniform sampler2D ParticlesSampler;
uniform sampler2D ParticlesDepthSampler;
uniform sampler2D WeatherSampler;
uniform sampler2D WeatherDepthSampler;
uniform sampler2D CloudsSampler;
uniform sampler2D CloudsDepthSampler;

uniform vec2 OutSize;

in vec2 texCoord;

#define NUM_LAYERS 6

vec4 color_layers[NUM_LAYERS];
float depth_layers[NUM_LAYERS];
int active_layers = 0;

out vec4 fragColor;

void try_insert( vec4 color, float depth ) {
    if ( color.a == 0.0 ) {
        return;
    }

    color_layers[active_layers] = color;
    depth_layers[active_layers] = depth;

    int jj = active_layers++;
    int ii = jj - 1;
    while ( jj > 0 && depth_layers[jj] > depth_layers[ii] ) {
        float depthTemp = depth_layers[ii];
        depth_layers[ii] = depth_layers[jj];
        depth_layers[jj] = depthTemp;

        vec4 colorTemp = color_layers[ii];
        color_layers[ii] = color_layers[jj];
        color_layers[jj] = colorTemp;

        jj = ii--;
    }
}

vec3 blend( vec3 dst, vec4 src ) {
    return ( dst * ( 1.0 - src.a ) ) + src.rgb;
}

float outline(sampler2D tex, vec2 uv, float depth) {
    float worldDepth = 2.0 * 0.05 * 1000.0 / (1000.0 + 0.05 - (depth * 2.0 - 1.0) * (1000.0 - 0.05));

    float d = max(outlineThickness * max((onePixelDistance - worldDepth) / onePixelDistance, 0.0), 1.0 / OutSize.y);
    float s00 = texture(tex, uv + vec2(-d * OutSize.y / OutSize.x, -d)).r;
    float s01 = texture(tex, uv + vec2(-d * OutSize.y / OutSize.x, +d)).r;
    float s11 = texture(tex, uv + vec2(+d * OutSize.y / OutSize.x, +d)).r;
    float s10 = texture(tex, uv + vec2(+d * OutSize.y / OutSize.x, -d)).r;

    float s = max(s00, max(s01, max(s11, s10)));
    float f = 2.0 * 0.05 * 1000.0 / (1000.0 + 0.05 - (s * 2.0 - 1.0) * (1000.0 - 0.05));
    float delta = f - worldDepth;

    return clamp(pow(max(delta, 0.0) * 0.15, 2.0), 0.0, 1.0) * exp(-worldDepth * distanceFalloff);
}

void main() {

    vec4 diffuse = vec4( texture( DiffuseSampler, texCoord ).rgb, 1.0 );
    float diffuseDepth = texture( DiffuseDepthSampler, texCoord ).r;

    vec3 colorOutline = vec3(0.0, 0.0, 0.0); // rgb settings of outline 1 is = to 255 0 is 0

    diffuse.rgb = mix(diffuse.rgb, colorOutline, outline(DiffuseDepthSampler, texCoord, diffuseDepth));

    color_layers[0] = diffuse;
    depth_layers[0] = diffuseDepth;
    active_layers = 1;

    try_insert( texture( TranslucentSampler, texCoord ), texture( TranslucentDepthSampler, texCoord ).r );
    try_insert( texture( ItemEntitySampler, texCoord ), texture( ItemEntityDepthSampler, texCoord ).r );
    try_insert( texture( ParticlesSampler, texCoord ), texture( ParticlesDepthSampler, texCoord ).r );
    try_insert( texture( WeatherSampler, texCoord ), texture( WeatherDepthSampler, texCoord ).r );
    try_insert( texture( CloudsSampler, texCoord ), texture( CloudsDepthSampler, texCoord ).r );

    vec3 texelAccum = color_layers[0].rgb;
    for ( int ii = 1; ii < active_layers; ++ii ) {
        texelAccum = blend( texelAccum, color_layers[ii] );
    }

    fragColor = vec4( texelAccum.rgb, 1.0 );
}
